# -*- coding: utf-8 -*-
from Acquisition import aq_parent
from OFS.interfaces import ITraversable
from Products.CMFCore.utils import getToolByName
from plone.app.blocks.interfaces import DEFAULT_AJAX_LAYOUT_REGISTRY_KEY
from plone.app.blocks.interfaces import DEFAULT_SITE_LAYOUT_REGISTRY_KEY
from plone.app.blocks.interfaces import SITE_LAYOUT_FILE_NAME
from plone.app.blocks.interfaces import SITE_LAYOUT_MANIFEST_FORMAT
from plone.app.blocks.interfaces import SITE_LAYOUT_RESOURCE_NAME
from plone.app.blocks.layoutbehavior import SiteLayoutView
from plone.app.blocks.utils import getDefaultAjaxLayout
from plone.app.blocks.utils import getDefaultSiteLayout
from plone.app.blocks.utils import getLayoutAwareSiteLayout
from plone.app.blocks.utils import resolveResource
from plone.memoize import view
from plone.memoize import volatile
from plone.registry.interfaces import IRecordModifiedEvent
from plone.resource.manifest import getAllResources
from plone.resource.traversal import ResourceTraverser
from plone.subrequest import ISubRequest
from zExceptions import NotFound
from zope.annotation import IAnnotations
from zope.component import adapter
from zope.globalrequest import getRequest
from zope.interface import implements
from zope.publisher.browser import BrowserView
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary
from zope.site.hooks import getSite
import Globals
import urlparse


class SiteLayoutTraverser(ResourceTraverser):
    """The site layout traverser.

    Allows traveral to /++sitelayout++<name> using ``plone.resource`` to fetch
    things stored either on the filesystem or in the ZODB.
    """

    name = SITE_LAYOUT_RESOURCE_NAME


class AnnotationsDict(dict):
    """Volatile annotations dictionary to pass to view.memoize_contextless when
    request thread local is not set"""
    implements(IAnnotations)


class _AvailableLayoutsVocabulary(object):
    """Vocabulary to return request cached available layouts of a given type
    """

    implements(IVocabularyFactory)

    def __init__(self, format, defaultFilename):
        self.format = format
        self.defaultFilename = defaultFilename
        self.request = getRequest() or AnnotationsDict()

    @view.memoize_contextless
    def __call__(self, context):
        items = {}  # dictionary is used here to avoid duplicate tokens

        resources = getAllResources(self.format)
        for name, manifest in resources.items():
            title = name.capitalize().replace('-', ' ').replace('.', ' ')
            filename = self.defaultFilename

            if manifest is not None:
                title = manifest['title'] or title
                filename = manifest['file'] or filename
                variants = manifest.get('variants') or []
            else:
                variants = []

            path = "/++%s++%s/%s" % (self.format.resourceType, name, filename)
            items[name] = SimpleTerm(path, name, title)

            for key, value in dict(variants).items():
                key_ = title_ = key.capitalize().replace('_', ' ')
                name_ = '{0:s}-{1:s}'.format(name, key)
                if manifest is not None and manifest['title']:
                    title_ = u'{0:s} ({1:s})'.format(title, key_)
                path = "/++%s++%s/%s" % (self.format.resourceType, name, value)
                items[name_] = SimpleTerm(path, name_, title_)

        items = sorted(items.values(), key=lambda term: term.title)
        return SimpleVocabulary(items)


class AvailableLayoutsVocabulary(object):
    """Vocabulary to return available layouts of a given type
    """

    implements(IVocabularyFactory)

    def __init__(self, format, defaultFilename):
        self.format = format
        self.defaultFilename = defaultFilename

    def __call__(self, context):
        # Instantiate the factory impl per call to support caching by request
        fab = _AvailableLayoutsVocabulary(self.format, self.defaultFilename)
        return fab(context)


AvailableSiteLayoutsVocabularyFactory = AvailableLayoutsVocabulary(
    SITE_LAYOUT_MANIFEST_FORMAT,
    SITE_LAYOUT_FILE_NAME,
)


def cacheKey(method, self):
    """Invalidate if the fti is modified, the global registry is modified,
    or the content is modified
    """

    if Globals.DevelopmentMode:
        raise volatile.DontCache()

    catalog = getToolByName(self.context, 'portal_catalog')

    return (
        getattr(self.context, '_p_mtime', None),
        self.request.form.get('ajax_load'),
        catalog.getCounter(),
    )


@adapter(IRecordModifiedEvent)
def globalSiteLayoutModified(event):
    """Invalidate caches if the global site layout is changed. This will
    likely also affect things cached using plone.app.caching, which is what
    we want - the page has probably changed
    """
    if event.record.__name__ in (DEFAULT_SITE_LAYOUT_REGISTRY_KEY,
                                 DEFAULT_AJAX_LAYOUT_REGISTRY_KEY):
        if event.oldValue != event.newValue:
            catalog = getToolByName(getSite(), 'portal_catalog', None)
            if catalog is not None and hasattr(catalog, '_increment_counter'):
                catalog._increment_counter()


class DefaultSiteLayout(BrowserView):
    """Look up and render the site layout to use for the context.

    Use this for a page that does not have the ILayout behavior, or a
    standalone page template.

    The idea is that you can do:

        <html data-layout="./@@default-site-layout">

    and always get the correct site layout for the page, taking section-
    specific settings into account.
    """

    def __call__(self):
        try:
            return self.index()
        except NotFound:
            if ISubRequest.providedBy(self.request):
                return SiteLayoutView(
                    self.context, self.request.PARENT_REQUEST)()
            else:
                return SiteLayoutView(self.context, self.request)()

    @property
    @volatile.cache(cacheKey, volatile.store_on_context)
    def layout(self):
        layout = self._getLayout()
        if layout is None:
            raise NotFound("No default site layout set")

        pathContext = self.context
        while not ITraversable.providedBy(pathContext):
            pathContext = aq_parent(pathContext)
            if pathContext is None:
                break

        path = layout
        if pathContext is not None:
            path = urlparse.urljoin(pathContext.absolute_url_path(), layout)

        return path

    @volatile.cache(cacheKey, volatile.store_on_context)
    def index(self):
        return resolveResource(self.layout)

    def _getLayout(self):
        if self.request.form.get('ajax_load'):
            return getDefaultAjaxLayout(self.context)
        else:
            return getDefaultSiteLayout(self.context)


class PageSiteLayout(DefaultSiteLayout):
    """Look up and render the site layout to use for the context.

    Use this for a page that does have the ILayout behavior. It will take the
    ``pageSiteLayout`` property into account.

    The idea is that you can do:

        <html data-layout="./@@page-site-layout">

    and always get the correct site layout for the page, taking section-
    and page-specific settings into account.
    """

    def _getLayout(self):
        if self.request.form.get('ajax_load'):
            return getDefaultAjaxLayout(self.context)
        else:
            return getLayoutAwareSiteLayout(self.context)



# -*- coding: utf-8 -*-
from plone.app.blocks.layoutbehavior import ContentLayoutView
from plone.app.blocks.layoutbehavior import ILayoutAware
from plone.app.blocks.testing import BLOCKS_FUNCTIONAL_TESTING
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.registry.interfaces import IRegistry
from plone.uuid.interfaces import IUUID
from zope.component import adapts
from zope.component import getGlobalSiteManager
from zope.component import getUtility
from zope.interface import implements
import pkg_resources
import unittest

try:
    pkg_resources.get_distribution('plone.app.contenttypes')
except pkg_resources.DistributionNotFound:
    HAS_PLONE_APP_CONTENTTYPES = False
else:
    HAS_PLONE_APP_CONTENTTYPES = True


class TestLayoutBehavior(unittest.TestCase):

    layer = BLOCKS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.registry = getUtility(IRegistry)
        setRoles(self.portal, TEST_USER_ID, ('Manager',))
        self.portal.invokeFactory('Folder', 'f1', title=u"Folder 1")
        self.portal['f1'].invokeFactory('Document', 'd1', title=u"Document 1")
        setRoles(self.portal, TEST_USER_ID, ('Member',))

    def test_content_layout_view_outputfilters(self):
        if HAS_PLONE_APP_CONTENTTYPES:
            from plone.app.contenttypes.interfaces import IDocument
            iface = IDocument
        else:
            iface = self.portal['f1']['d1'].__class__

        class DocumentLayoutAware(object):
            implements(ILayoutAware)
            adapts(iface)

            def __init__(self, context):
                self.context = context

            content = u'<html><body><a href="{0:s}"></a></body></html>'.format(
                'resolveuid/{0:s}'.format(IUUID(self.portal['f1'])))
            sectionSiteLayout = None
            pageSiteLayout = None

        sm = getGlobalSiteManager()
        sm.registerAdapter(DocumentLayoutAware)

        registrations = sm.getAdapters((self.portal['f1']['d1'],),
                                       ILayoutAware)
        self.assertEqual(len(list(registrations)), 1)

        view = ContentLayoutView(self.portal['f1']['d1'], self.request)
        rendered = view()

        sm.unregisterAdapter(DocumentLayoutAware)

        self.assertNotIn(IUUID(self.portal['f1']), rendered)
        self.assertIn(self.portal['f1'].absolute_url(), rendered)



# -*- coding: utf-8 -*-
from plone.app.blocks.testing import BLOCKS_INTEGRATION_TESTING

import unittest


class TestTraversers(unittest.TestCase):

    layer = BLOCKS_INTEGRATION_TESTING

    def test_site_layout_traverser_registered(self):
        from plone.resource.file import FilesystemFile
        portal = self.layer['portal']

        layout = portal.restrictedTraverse(
            '++sitelayout++testlayout1/site.html')
        self.assertTrue(isinstance(layout, FilesystemFile))

    def test_site_layouts_vocabulary_and_manifest(self):
        from zope.schema.vocabulary import getVocabularyRegistry
        portal = self.layer['portal']

        vocab = getVocabularyRegistry().get(portal,
                                            'plone.availableSiteLayouts')
        vocab = list(vocab)
        vocab.sort(key=lambda t: t.token)

        self.assertEqual(len(vocab), 2)

        self.assertEqual(vocab[0].token, 'testlayout1')
        self.assertEqual(vocab[0].title, 'Testlayout1')
        self.assertEqual(vocab[0].value,
                         u'/++sitelayout++testlayout1/site.html')

        self.assertEqual(vocab[1].token, 'testlayout2')
        self.assertEqual(vocab[1].title, 'My site layout')
        self.assertEqual(vocab[1].value,
                         u'/++sitelayout++testlayout2/mylayout.html')



# -*- coding: utf-8 -*-
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.testing import Layer
from zope.configuration import xmlconfig
import pkg_resources

try:
    pkg_resources.get_distribution('plone.app.contenttypes')
except pkg_resources.DistributionNotFound:
    HAS_PLONE_APP_CONTENTTYPES = False
else:
    HAS_PLONE_APP_CONTENTTYPES = True

try:
    pkg_resources.get_distribution('plone.app.theming')
except pkg_resources.DistributionNotFound:
    HAS_PLONE_APP_THEMING = False
else:
    HAS_PLONE_APP_THEMING = True


class BlocksLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # load ZCML
        if HAS_PLONE_APP_CONTENTTYPES:
            import plone.app.contenttypes
            self.loadZCML(package=plone.app.contenttypes)
        import plone.app.blocks
        self.loadZCML(package=plone.app.blocks)

        # Register directory for testing
        xmlconfig.string("""\
<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="plone"
    package="plone.app.blocks.tests">

    <plone:static
        type="sitelayout"
        name="testlayout1"
        directory="resources/sitelayout/testlayout1"
        />

    <plone:static
        type="sitelayout"
        name="testlayout2"
        directory="resources/sitelayout/testlayout2"
        />

</configure>
""", context=configurationContext)
        if 'virtual_hosting' not in app.objectIds():
            # If ZopeLite was imported, we have no default virtual
            # host monster
            from Products.SiteAccess.VirtualHostMonster \
                import manage_addVirtualHostMonster
            manage_addVirtualHostMonster(app, 'virtual_hosting')

    def setUpPloneSite(self, portal):
        # ensure plone.app.theming disabled
        if HAS_PLONE_APP_THEMING:
            from plone.registry.interfaces import IRegistry
            from zope.component import getUtility
            registry = getUtility(IRegistry)
            key = 'plone.app.theming.interfaces.IThemeSettings.enabled'
            if key in registry:
                registry[key] = False
        # install plone.app.contenttypes on Plone 5
        if HAS_PLONE_APP_CONTENTTYPES:
            self.applyProfile(portal, 'plone.app.contenttypes:default')
        # install into the Plone site
        self.applyProfile(portal, 'plone.app.blocks:default')


class PrettyPrintLayer(Layer):

    def setUp(self):
        from plone.app.blocks.transform import ParseXML
        ParseXML.pretty_print = True

    def tearDown(self):
        from plone.app.blocks.transform import ParseXML
        ParseXML.pretty_print = False

BLOCKS_FIXTURE = BlocksLayer()
PRETTY_PRINT_FIXTURE = PrettyPrintLayer()
BLOCKS_INTEGRATION_TESTING = IntegrationTesting(
    bases=(BLOCKS_FIXTURE,), name="Blocks:Integration")
BLOCKS_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(BLOCKS_FIXTURE,), name="Blocks:Functional")
BLOCKS_FUNCTIONAL_TESTING_PRETTY_PRINT = FunctionalTesting(
    bases=(PRETTY_PRINT_FIXTURE, BLOCKS_FIXTURE,),
    name="Blocks:Functional Pretty Printing")



# -*- coding: utf-8 -*-
from plone.app.blocks.interfaces import DEFAULT_SITE_LAYOUT_REGISTRY_KEY
from plone.app.blocks.layoutbehavior import ILayoutAware
from plone.app.blocks.layoutbehavior import SiteLayoutView
from plone.app.blocks.testing import BLOCKS_FUNCTIONAL_TESTING
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.registry.interfaces import IRegistry
from zExceptions import NotFound
from zope.interface import implements
from zope.component import adapts
from zope.component import getMultiAdapter
from zope.component import getGlobalSiteManager
from zope.component import getUtility

import pkg_resources
import transaction
import unittest

try:
    pkg_resources.get_distribution('plone.app.contenttypes')
except pkg_resources.DistributionNotFound:
    HAS_PLONE_APP_CONTENTTYPES = False
else:
    HAS_PLONE_APP_CONTENTTYPES = True


class TestPageSiteLayout(unittest.TestCase):

    layer = BLOCKS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.registry = getUtility(IRegistry)
        setRoles(self.portal, TEST_USER_ID, ('Manager',))
        self.portal.invokeFactory('Folder', 'f1', title=u"Folder 1")
        self.portal['f1'].invokeFactory('Document', 'd1', title=u"Document 1")
        setRoles(self.portal, TEST_USER_ID, ('Member',))

    def test_page_site_layout_no_registry_key(self):
        self.registry[DEFAULT_SITE_LAYOUT_REGISTRY_KEY] = None
        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        self.assertRaises(NotFound, view.index)

        default_view = SiteLayoutView(self.portal['f1']['d1'], self.request)
        self.assertEqual(view().split(), default_view().split())

    def test_page_site_layout_default(self):
        self.registry[DEFAULT_SITE_LAYOUT_REGISTRY_KEY] =\
            '/++sitelayout++testlayout1/site.html'
        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        rendered = view()
        self.assertTrue(u"Layout title" in rendered)

    def test_page_site_layout_page_override(self):
        self.registry[DEFAULT_SITE_LAYOUT_REGISTRY_KEY] =\
            '/++sitelayout++testlayout1/site.html'

        if HAS_PLONE_APP_CONTENTTYPES:
            from plone.app.contenttypes.interfaces import IDocument
            iface = IDocument
        else:
            iface = self.portal['f1']['d1'].__class__

        class DocumentLayoutAware(object):
            implements(ILayoutAware)
            adapts(iface)

            def __init__(self, context):
                self.context = context

            content = u"<html><body>N/A</body></html>"
            sectionSiteLayout = None
            pageSiteLayout = '/++sitelayout++testlayout2/mylayout.html'

        sm = getGlobalSiteManager()
        sm.registerAdapter(DocumentLayoutAware)

        registrations = sm.getAdapters((self.portal['f1']['d1'],),
                                       ILayoutAware)
        self.assertEqual(len(list(registrations)), 1)

        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        rendered = view()

        sm.unregisterAdapter(DocumentLayoutAware)

        self.assertFalse(u"Layout title" in rendered)
        self.assertTrue(u"Layout 2 title" in rendered)

    def test_page_site_layout_section_override(self):
        self.registry[DEFAULT_SITE_LAYOUT_REGISTRY_KEY] =\
            '/++sitelayout++testlayout1/site.html'

        class FolderLayoutAware(object):
            implements(ILayoutAware)
            adapts(self.portal['f1'].__class__)

            def __init__(self, context):
                self.context = context

            content = u"<html><body>N/A</body></html>"
            sectionSiteLayout = '/++sitelayout++testlayout2/mylayout.html'
            pageSiteLayout = None

        sm = getGlobalSiteManager()
        sm.registerAdapter(FolderLayoutAware)

        registrations = sm.getAdapters((self.portal['f1'],), ILayoutAware)
        self.assertEqual(len(list(registrations)), 1)

        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        rendered = view()

        sm.unregisterAdapter(FolderLayoutAware)

        self.assertFalse(u"Layout title" in rendered)
        self.assertTrue(u"Layout 2 title" in rendered)

    def test_page_site_layout_cache(self):
        self.registry[DEFAULT_SITE_LAYOUT_REGISTRY_KEY] =\
            '/++sitelayout++testlayout1/site.html'

        currentSectionSiteLayout = '/++sitelayout++testlayout2/mylayout.html'

        class FolderLayoutAware(object):
            implements(ILayoutAware)
            adapts(self.portal['f1'].__class__)

            def __init__(self, context):
                self.context = context

            content = u"<html><body>N/A</body></html>"
            pageSiteLayout = None

            @property
            def sectionSiteLayout(self):
                return currentSectionSiteLayout

        sm = getGlobalSiteManager()
        sm.registerAdapter(FolderLayoutAware)

        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        rendered = view()

        self.assertFalse(u"Layout title" in rendered)
        self.assertTrue(u"Layout 2 title" in rendered)

        # Change the section value
        currentSectionSiteLayout = '/++sitelayout++testlayout1/site.html'

        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        rendered = view()

        sm.unregisterAdapter(FolderLayoutAware)

        # Cache means our change is ignored
        self.assertFalse(u"Layout title" in rendered)
        self.assertTrue(u"Layout 2 title" in rendered)

    def test_page_site_layout_cache_invalidate_mtime(self):

        self.registry[DEFAULT_SITE_LAYOUT_REGISTRY_KEY] =\
            '/++sitelayout++testlayout1/site.html'

        currentSectionSiteLayout = '/++sitelayout++testlayout2/mylayout.html'

        class FolderLayoutAware(object):
            implements(ILayoutAware)
            adapts(self.portal['f1'].__class__)

            def __init__(self, context):
                self.context = context

            content = u"<html><body>N/A</body></html>"
            pageSiteLayout = None

            @property
            def sectionSiteLayout(self):
                return currentSectionSiteLayout

        sm = getGlobalSiteManager()
        sm.registerAdapter(FolderLayoutAware)

        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        rendered = view()

        self.assertFalse(u"Layout title" in rendered)
        self.assertTrue(u"Layout 2 title" in rendered)

        # Trigger invalidation by modifying the context
        self.portal['f1']['d1'].title = u"New title"
        transaction.commit()

        # Change the section value
        currentSectionSiteLayout = '/++sitelayout++testlayout1/site.html'

        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        rendered = view()

        sm.unregisterAdapter(FolderLayoutAware)

        # We now get the new layout
        self.assertTrue(u"Layout title" in rendered)
        self.assertFalse(u"Layout 2 title" in rendered)

    def test_page_site_layout_cache_invalidate_catalog_counter(self):
        self.registry[DEFAULT_SITE_LAYOUT_REGISTRY_KEY] =\
            '/++sitelayout++testlayout1/site.html'

        currentSectionSiteLayout = '/++sitelayout++testlayout2/mylayout.html'

        class FolderLayoutAware(object):
            implements(ILayoutAware)
            adapts(self.portal['f1'].__class__)

            def __init__(self, context):
                self.context = context

            content = u"<html><body>N/A</body></html>"
            pageSiteLayout = None

            @property
            def sectionSiteLayout(self):
                return currentSectionSiteLayout

        sm = getGlobalSiteManager()
        sm.registerAdapter(FolderLayoutAware)

        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        rendered = view()

        self.assertFalse(u"Layout title" in rendered)
        self.assertTrue(u"Layout 2 title" in rendered)

        # Trigger invalidation by incrementing the catalog counter
        self.portal['portal_catalog']._increment_counter()

        # Change the section value
        currentSectionSiteLayout = '/++sitelayout++testlayout1/site.html'

        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        rendered = view()

        sm.unregisterAdapter(FolderLayoutAware)

        # We now get the new layout
        self.assertTrue(u"Layout title" in rendered)
        self.assertFalse(u"Layout 2 title" in rendered)

    def test_page_site_layout_cache_invalidate_registry_key(self):
        self.registry[DEFAULT_SITE_LAYOUT_REGISTRY_KEY] =\
            '/++sitelayout++testlayout1/site.html'

        currentSectionSiteLayout = '/++sitelayout++testlayout2/mylayout.html'

        class FolderLayoutAware(object):
            implements(ILayoutAware)
            adapts(self.portal['f1'].__class__)

            def __init__(self, context):
                self.context = context

            content = u"<html><body>N/A</body></html>"
            pageSiteLayout = None

            @property
            def sectionSiteLayout(self):
                return currentSectionSiteLayout

        sm = getGlobalSiteManager()
        sm.registerAdapter(FolderLayoutAware)

        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        rendered = view()

        self.assertFalse(u"Layout title" in rendered)
        self.assertTrue(u"Layout 2 title" in rendered)

        # Trigger invalidation by modifying the global registry key
        self.registry[DEFAULT_SITE_LAYOUT_REGISTRY_KEY] =\
            '/++sitelayout++testlayout2/mylayout.html'

        # Change the section value
        currentSectionSiteLayout = '/++sitelayout++testlayout1/site.html'

        view = getMultiAdapter((self.portal['f1']['d1'], self.request,),
                               name=u'page-site-layout')
        rendered = view()

        sm.unregisterAdapter(FolderLayoutAware)

        # We now get the new layout
        self.assertTrue(u"Layout title" in rendered)
        self.assertFalse(u"Layout 2 title" in rendered)



# -*- coding: utf-8 -*-
from plone.transformchain.zpublisher import applyTransform
from plone.app.blocks.interfaces import IBlocksLayer
from plone.app.blocks.interfaces import IBlocksTransformEnabled
from plone.app.blocks.testing import BLOCKS_INTEGRATION_TESTING
from zope.interface import alsoProvides
from zope.interface import implements

import unittest


class TestTransforms(unittest.TestCase):

    layer = BLOCKS_INTEGRATION_TESTING

    def test_transforms_with_crlf(self):
        """Test fix for issue where layouts with CR[+LF] line-endings are
        somehow turned into having &#13; line-endings and getting their heads
        being dropped
        """

        class TransformedView(object):
            implements(IBlocksTransformEnabled)

            def __init__(self, ret_body):
                self.__call__ = lambda b=ret_body: b

        body = """\
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">&#13;
<head></head>&#13;
<body></body>&#13;
</html>"""
        request = self.layer['request']
        request.set('PUBLISHED', TransformedView(body))
        request.response.setBase(request.getURL())
        request.response.setHeader('content-type', 'text/html')
        request.response.setBody(body)

        alsoProvides(request, IBlocksLayer)
        result = applyTransform(request)
        self.assertIn('<head>', ''.join(result))

    def test_transforms_with_cdata(self):
        """Test fix for issue where layouts with inline js got rendered with
        quoted (and therefore broken) <![CDATA[...]]> block
        """

        class TransformedView(object):
            implements(IBlocksTransformEnabled)

            def __init__(self, ret_body):
                self.__call__ = lambda b=ret_body: b

        body = """\
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head><script type="text/javascript"><![CDATA[]]></script></head>
<body></body>
</html>"""
        request = self.layer['request']
        request.set('PUBLISHED', TransformedView(body))
        request.response.setBase(request.getURL())
        request.response.setHeader('content-type', 'text/html')
        request.response.setBody(body)

        alsoProvides(request, IBlocksLayer)
        result = applyTransform(request)
        self.assertIn('<![CDATA[]]>', ''.join(result))



# -*- coding: utf-8 -*-
from plone.registry import Record
from plone.registry import field
from plone.registry.interfaces import IRegistry
from zope.component import getUtility


def step_setup_various(context):
    if context.readDataFile('plone.app.blocks_default.txt') is None:
        return
    portal = context.getSite()
    initialize_default_layout_registry_values(portal)


def initialize_default_layout_registry_values(portal):
    registry = getUtility(IRegistry)
    records = (
        ('plone.defaultSiteLayout', u'Default site layout',
         u'The default site layout for the site', None),
        ('plone.defaultAjaxLayout', u'Default ajax layout',
         u'The default ajax layout for the site', None),
    )
    for key, title, description, value in records:
        if key not in registry.records:
            registry.records[key] = Record(
                field.BytesLine(
                    title=title,
                    description=description,
                    required=False
                ), value)



# -*- coding: utf-8 -*-
from copy import deepcopy
import logging

from AccessControl import getSecurityManager
from Acquisition import aq_inner
from Acquisition import aq_parent
from diazo import rules
from diazo import cssrules
from diazo import compiler
from lxml import etree
from lxml import html
from diazo import utils
from plone.memoize import ram
from plone.memoize.volatile import DontCache
from plone.registry.interfaces import IRegistry
from zExceptions import NotFound
from z3c.form.interfaces import IFieldWidget
from zope.component import getMultiAdapter
from zope.component import queryUtility
from zope.security.interfaces import IPermission
from zope.site.hooks import getSite
import Globals

from plone.app.blocks.interfaces import DEFAULT_SITE_LAYOUT_REGISTRY_KEY
from plone.app.blocks.interfaces import DEFAULT_AJAX_LAYOUT_REGISTRY_KEY
from plone.app.blocks.layoutbehavior import ILayoutAware
from plone.subrequest import subrequest


headXPath = etree.XPath("/html/head")
layoutAttrib = 'data-layout'
layoutXPath = etree.XPath("/html/@" + layoutAttrib)
gridAttrib = 'data-gridsystem'
gridXPath = etree.XPath("/html/@" + gridAttrib)
tileAttrib = 'data-tile'
tileRulesAttrib = 'data-rules'
headTileXPath = etree.XPath("/html/head//*[@" + tileAttrib + "]")
bodyTileXPath = etree.XPath("/html/body//*[@" + tileAttrib + "]")
panelXPath = etree.XPath("//*[@data-panel]")
gridDataAttrib = 'data-grid'
gridDataXPath = etree.XPath("//*[@" + gridDataAttrib + "]")
logger = logging.getLogger('plone.app.blocks')


def extractCharset(response, default='utf-8'):
    """Get the charset of the given response
    """

    charset = default
    if 'content-type' in response.headers:
        for item in response.headers['content-type'].split(';'):
            if item.strip().startswith('charset'):
                charset = item.split('=')[1].strip()
                break
    return charset


def resolve(url):
    """Resolve the given URL to an lxml tree.
    """

    resolved = resolveResource(url)
    if not resolved.strip():
        return None
    try:
        if isinstance(resolved, unicode):
            html_parser = html.HTMLParser(encoding='utf-8')
            return html.fromstring(resolved.encode('utf-8'),
                                   parser=html_parser).getroottree()
        else:
            return html.fromstring(resolved).getroottree()
    except etree.XMLSyntaxError as e:
        logger.error('%s: %s' % (repr(e), url))
        return None


def resolveResource(url):
    """Resolve the given URL to a unicode string. If the URL is an absolute
    path, it will be made relative to the Plone site root.
    """
    if url.startswith('/'):
        site = getSite()
        url = '/'.join(site.getPhysicalPath()) + url

    response = subrequest(url)
    if response.status == 404:
        raise NotFound(url)

    resolved = response.getBody()

    if isinstance(resolved, str):
        charset = extractCharset(response)
        resolved = resolved.decode(charset)

    if response.status in (301, 302):
        site = getSite()
        location = response.headers.get('location') or ''
        if location.startswith(site.absolute_url()):
            return resolveResource(location[len(site.absolute_url()):])

    elif response.status != 200:
        raise RuntimeError(resolved)

    return resolved


def xpath1(xpath, node, strict=True):
    """Return a single node matched by the given etree.XPath object.
    """

    if isinstance(xpath, basestring):
        xpath = etree.XPath(xpath)

    result = xpath(node)
    if len(result) == 1:
        return result[0]
    else:
        if (len(result) > 1 and strict) or len(result) == 0:
            return None
        else:
            return result


def append_text(element, text):
    if text:
        element.text = (element.text or '') + text


def append_tail(element, text):
    if text:
        element.tail = (element.tail or '') + text


def replace_with_children(element, wrapper):
    """element.replace also replaces the tail and forgets the wrapper.text
    """
    # XXX needs tests
    parent = element.getparent()
    index = parent.index(element)
    if index == 0:
        previous = None
    else:
        previous = parent[index - 1]
    if wrapper is None:
        children = []
    else:
        if index == 0:
            append_text(parent, wrapper.text)
        else:
            append_tail(previous, wrapper.text)
        children = wrapper.getchildren()
    parent.remove(element)
    if not children:
        if index == 0:
            append_text(parent, element.tail)
        else:
            append_tail(previous, element.tail)
    else:
        append_tail(children[-1], element.tail)
        children.reverse()
        for child in children:
            parent.insert(index, child)


def replace_content(element, wrapper):
    """Similar to above but keeps parent tag
    """
    del element[:]
    if wrapper is not None:
        element.text = wrapper.text
        element.extend(wrapper.getchildren())


def getDefaultSiteLayout(context):
    """Get the path to the site layout to use by default for the given content
    object
    """

    # Note: the sectionSiteLayout on context is for pages *under* context, not
    # necessarily context itself

    parent = aq_parent(aq_inner(context))
    while parent is not None:
        layoutAware = ILayoutAware(parent, None)
        if layoutAware is not None:
            if getattr(layoutAware, 'sectionSiteLayout', None):
                return layoutAware.sectionSiteLayout
        parent = aq_parent(aq_inner(parent))

    registry = queryUtility(IRegistry)
    if registry is None:
        return None

    return registry.get(DEFAULT_SITE_LAYOUT_REGISTRY_KEY)


def getDefaultAjaxLayout(context):
    """Get the path to the ajax site layout to use by default for the given
    content object
    """

    registry = queryUtility(IRegistry)
    if registry is not None:
        return registry.get(DEFAULT_AJAX_LAYOUT_REGISTRY_KEY)
    else:
        return getLayoutAwareSiteLayout(context)


def getLayoutAwareSiteLayout(context):
    """Get the path to the site layout for a page. This is generally only
    appropriate for the view of this page. For a generic template or view, use
    getDefaultSiteLayout(context) instead.
    """

    layoutAware = ILayoutAware(context, None)
    if layoutAware is not None:
        if getattr(layoutAware, 'pageSiteLayout', None):
            return layoutAware.pageSiteLayout

    return getDefaultSiteLayout(context)


class PermissionChecker(object):

    def __init__(self, permissions, context):
        self.permissions = permissions
        self.context = context
        self.sm = getSecurityManager()
        self.cache = {}

    def allowed(self, field_name):
        permission_name = self.permissions.get(field_name, None)
        if permission_name is not None:
            if permission_name not in self.cache:
                permission = queryUtility(IPermission, name=permission_name)
                if permission is None:
                    self.cache[permission_name] = True
                else:
                    self.cache[permission_name] = bool(
                        self.sm.checkPermission(permission.title,
                                                self.context),
                    )
        return self.cache.get(permission_name, True)


def _getWidgetName(field, widgets, request):
    if field.__name__ in widgets:
        factory = widgets[field.__name__]
    else:
        factory = getMultiAdapter((field, request), IFieldWidget)
    if isinstance(factory, basestring):
        return factory
    if not isinstance(factory, type):
        factory = factory.__class__
    return '%s.%s' % (factory.__module__, factory.__name__)


def isVisible(name, omitted):
    value = omitted.get(name, False)
    if isinstance(value, basestring):
        return value == 'false'
    else:
        return not bool(value)


def add_theme(rules_doc, theme_doc, absolute_prefix=None):
    if absolute_prefix is None:
        absolute_prefix = ''
    root = rules_doc.getroot()
    element = root.makeelement(
        rules.fullname(rules.namespaces['diazo'], 'theme'))
    root.append(element)
    rules.expand_theme(element, theme_doc, absolute_prefix)
    return rules_doc


def process_rules(rules_doc, theme=None, trace=None, css=True,
                  absolute_prefix=None, includemode=None,
                  update=True, stop=None):
    if trace:
        trace = '1'
    else:
        trace = '0'
    if stop == 0:
        return rules_doc
    if stop == 1:
        return rules_doc
    rules_doc = rules.add_identifiers(rules_doc)
    if stop == 2 or stop == 'add_identifiers':
        return rules_doc
    if update:
        rules_doc = rules.update_namespace(rules_doc)
    if stop == 3:
        return rules_doc
    if css:
        rules_doc = cssrules.convert_css_selectors(rules_doc)
    if stop == 4:
        return rules_doc
    rules_doc = rules.fixup_theme_comment_selectors(rules_doc)
    if stop == 5:
        return rules_doc
    if theme is not None:
        rules_doc = add_theme(rules_doc, theme, absolute_prefix)
    if stop == 6:
        return rules_doc
    if includemode is None:
        includemode = 'document'
    includemode = "'%s'" % includemode
    rules_doc = rules.normalize_rules(rules_doc, includemode=includemode)
    if stop == 7:
        return rules_doc
    rules_doc = rules.apply_conditions(rules_doc)
    if stop == 8:
        return rules_doc
    rules_doc = rules.merge_conditions(rules_doc)
    if stop == 9:
        return rules_doc
    rules_doc = rules.fixup_themes(rules_doc)
    if stop == 10:
        return rules_doc
    rules_doc = rules.annotate_themes(rules_doc)
    if stop == 11:
        return rules_doc
    rules_doc = rules.annotate_rules(rules_doc)
    if stop == 12:
        return rules_doc
    rules_doc = rules.apply_rules(rules_doc, trace=trace)
    return rules_doc


def compile_theme(rules_doc, theme_doc=None, css=True,
                  absolute_prefix=None, update=True, trace=False,
                  includemode=None, parser=None, compiler_parser=None):
    rules_doc = process_rules(
        rules_doc=rules_doc,
        theme=theme_doc,
        css=css,
        absolute_prefix=absolute_prefix,
        update=update,
        trace=trace,
        includemode=includemode,
    )

    # Build a document with all the <xsl:param /> values to set the defaults
    # for every value passed in as xsl_params
    known_params = compiler.build_xsl_params_document({})

    # Create a pseudo resolver for this
    known_params_url = 'file:///__diazo_known_params__'
    emit_stylesheet_resolver = utils.CustomResolver({
        known_params_url: etree.tostring(known_params)})
    emit_stylesheet_parser = etree.XMLParser()
    emit_stylesheet_parser.resolvers.add(emit_stylesheet_resolver)

    # Run the final stage compiler
    emit_stylesheet = utils.pkg_xsl(
        'emit-stylesheet.xsl', parser=emit_stylesheet_parser)
    compiled_doc = emit_stylesheet(rules_doc)
    compiled_doc = compiler.set_parser(
        etree.tostring(compiled_doc), parser, compiler_parser)

    return compiled_doc


def cacheKey(func, rules_url, theme_node):
    if Globals.DevelopmentMode:
        raise DontCache()
    return ':'.join([rules_url, html.tostring(theme_node)])


@ram.cache(cacheKey)
def resolve_transform(rules_url, theme_node):
    rules_doc = resolveResource(rules_url)  # may raise NotFound
    rules_doc = etree.ElementTree(etree.fromstring(rules_doc))
    compiled = compile_theme(rules_doc,
                             etree.ElementTree(deepcopy(theme_node)))
    transform = etree.XSLT(compiled)
    return transform



# -*- coding: utf-8 -*-
from plone.app.blocks import utils
from zope.component import getUtility
from zope.interface import Interface
from zope.interface import implements

import json


class IGridSystem(Interface):
    """ Utility to get the grid System
    """


class BS3GridSystem(object):
    implements(IGridSystem)

    def __init__(self):
        self.offset = 1

    def transform(self, key):
        """ its possible:
            {type: row} -> row
            {type: cell, info: {xs:False, sm:False, md:True, lg:true, pos:{x:1 width:10}}} ->
                hidden-xs hidden-sm col-md-10
        """
        element = json.loads(key)
        if 'type' in element and element['type'] == 'row':
            self.offset = 1
            return 'row'
        elif 'type' in element and element['type'] == 'cell':
            result = ''
            if 'info' in element:
                if 'xs' in element['info'] and element['info']['xs'].lower() == "false":
                    result += 'hidden-xs '
                if 'sm' in element['info'] and element['info']['sm'].lower() == "false":
                    result += 'hidden-sm '
                if 'md' in element['info'] and element['info']['md'].lower() == "false":
                    result += 'hidden-md '
                if 'lg' in element['info'] and element['info']['lg'].lower() == "false":
                    result += 'hidden-lg '
                if 'pos' in element['info']:
                    if element['info']['pos']['x'] > self.offset:
                        result += 'col-md-offset-%d ' % (element['info']['pos']['x'] - (self.offset - 1))
                    if 'width' in element['info']['pos']:
                        self.offset += element['info']['pos']['width']
                        result += 'col-md-%d' % element['info']['pos']['width']
            return result


class DecoGridSystem(object):
    implements(IGridSystem)

    def transform(self, key):
        """ its possible:
            {type: row} -> row
            {type: cell, info: {xs:False, sm:False, md:True, lg:true, pos:{x:1 width:10}}} ->
            cell position-1 width-10
        """
        element = json.loads(key)
        if 'type' in element and element['type'] == 'row':
            return 'row'
        elif 'type' in element and element['type'] == 'cell':
            result = 'cell '
            if 'info' in element:
                if 'pos' in element['info']:
                    if 'x' in element['info']['pos']:
                        deco_pos = int(element['info']['pos']['x']) - 1
                        result += 'position-%d ' % deco_pos
                    if 'width' in element['info']['pos']:
                        result += 'width-%d' % element['info']['pos']['width']
            return result


def merge(request, layoutTree):
    """Perform grid system merging for the given page.

    Returns None if the page has no layout.
    """

    # Find layout node
    gridSystem = utils.xpath1(utils.gridXPath, layoutTree)
    if gridSystem is None:
        gridSystem = 'deco'

    gridUtil = getUtility(IGridSystem, gridSystem)()
    for layoutGridNode in utils.gridDataXPath(layoutTree):
        gridinfo = layoutGridNode.attrib['data-grid']
        cssGridClass = gridUtil.transform(gridinfo)
        if cssGridClass is not None:
            if 'class' in layoutGridNode.attrib:
                layoutGridNode.attrib['class'] = layoutGridNode.attrib['class'] + ' ' + cssGridClass  # noqa
            else:
                layoutGridNode.attrib['class'] = cssGridClass
        del layoutGridNode.attrib['data-grid']
    return layoutTree



# -*- coding: utf-8 -*-
from plone.app.blocks.testing import BLOCKS_FIXTURE
from plone.app.blocks.tiles import renderTiles
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.tiles import Tile
from repoze.xmliter.utils import getHTMLSerializer
from zope import schema
from zope.configuration import xmlconfig
from zope.interface import Interface
from zope.interface import implementer
import unittest


class ITestTile(Interface):

    magicNumber = schema.Int(title=u"Magic number", required=False)


@implementer(ITestTile)
class TestTile(Tile):

    def __call__(self):
        # fake a page template to keep things simple in the test
        return """\
<html>
<head>
  <meta name="tile-name" content="%(name)s" />
</head>
<body>
  <p>
    This is a demo tile with id %(name)s
  </p>
  <p>
    Magic number: %(number)d; Form: %(form)s;
    Query string: %(queryString)s; URL: %(url)s
  </p>
</body>
</html>""" % dict(name=self.id, number=self.data['magicNumber'] or -1,
                  form=sorted(self.request.form.items()),
                  queryString=self.request['QUERY_STRING'],
                  url=self.request.getURL())


@implementer(ITestTile)
class TestTileBroken(TestTile):

    def __call__(self):
        raise Exception("This tile is broken.")


class TestTilesLayer(PloneSandboxLayer):

    defaultBases = (BLOCKS_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string("""\
<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="plone.app.blocks">

  <include package="plone.tiles" file="meta.zcml" />

  <plone:tile
      name="test.tile1"
      title="Test Tile"
      description=""
      add_permission="cmf.ModifyPortalContent"
      schema="plone.app.blocks.tests.test_tiles.ITestTile"
      class="plone.app.blocks.tests.test_tiles.TestTile"
      permission="zope2.View"
      for="*"
      />

  <plone:tile
      name="test.tile1.broken"
      title="Broken Test Tile"
      description=""
      add_permission="cmf.ModifyPortalContent"
      schema="plone.app.blocks.tests.test_tiles.ITestTile"
      class="plone.app.blocks.tests.test_tiles.TestTileBroken"
      permission="zope2.View"
      for="*"
      />

</configure>
""", context=configurationContext)

BLOCKS_TILES_FIXTURE = TestTilesLayer()

BLOCKS_TILES_INTEGRATION_TESTING = IntegrationTesting(
    bases=(BLOCKS_TILES_FIXTURE,), name="Blocks:Tiles:Integration")


testLayout1 = """\
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html data-layout="./@@default-site-layout">
<head></head>
<body>
  <h1>Welcome!</h1>
  <div data-panel="panel1">
    Page panel 1
    <div id="page-tile2" data-tile="./@@test.tile1/tile2?magicNumber:int=2">Page tile 2 placeholder</div>
  </div>
  <div data-panel="panel2">
    Page panel 2
    <div id="page-tile3" data-tile="./@@test.tile1/tile3">Page tile 3 placeholder</div>
  </div>
  <div data-panel="panel4">
    Page panel 4 (ignored)
    <div id="page-tile4" data-tile="./@@test.tile1/tile4">Page tile 4 placeholder</div>
  </div>
</body>
</html>
"""


testLayout2 = """\
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html data-layout="./@@default-site-layout">
<head></head>
<body>
  <h1>Welcome!</h1>
  <div data-panel="panel1">
    Page panel 1
    <div id="page-tile2" data-tile="./@@test.tile1/tile2?magicNumber:int=2">Page tile 2 placeholder</div>
  </div>
  <div data-panel="panel2">
    Page panel 2
    <div id="page-tile3" data-tile="./@@test.tile1.broken/tile3">Page tile 3 placeholder</div>
  </div>
  <div data-panel="panel4">
    Page panel 4 (ignored)
    <div id="page-tile4" data-tile="./@@test.tile1/tile4">Page tile 4 placeholder</div>
  </div>
</body>
</html>
"""


class TestRenderTiles(unittest.TestCase):

    layer = BLOCKS_TILES_INTEGRATION_TESTING

    def testRenderTiles(self):
        serializer = getHTMLSerializer([testLayout1])
        request = self.layer['request']
        tree = serializer.tree
        renderTiles(request, tree)
        result = serializer.serialize()
        self.assertIn('This is a demo tile with id tile2', result)
        self.assertIn('This is a demo tile with id tile3', result)
        self.assertIn('This is a demo tile with id tile4', result)

    def testRenderTilesError(self):
        serializer = getHTMLSerializer([testLayout2])
        request = self.layer['request']
        tree = serializer.tree
        renderTiles(request, tree)
        result = serializer.serialize()
        self.assertIn('This is a demo tile with id tile2', result)
        self.assertNotIn('This is a demo tile with id tile3', result)
        self.assertIn('There was an error while rendering this tile', result)
        self.assertIn('This is a demo tile with id tile4', result)



# -*- coding: utf-8 -*-
from plone.app.blocks import PloneMessageFactory
from plone.app.blocks import utils
from plone.app.blocks.interfaces import IBlocksSettings
from plone.app.blocks.utils import resolve_transform
from plone.registry.interfaces import IRegistry
from plone.tiles.interfaces import ESI_HEADER
from plone.tiles.interfaces import ESI_HEADER_KEY
from urlparse import urljoin
from zExceptions import NotFound
from zope.component import queryUtility
from zope.i18n import translate
from lxml import html


def errorTile(request):
    msg = PloneMessageFactory('There was an error while rendering this tile')
    translated = translate(msg, context=request)
    return html.fromstring(translated).getroottree()


def renderTiles(request, tree):
    """Find all tiles in the given response, contained in the lxml element
    tree `tree`, and insert them into the output.

    Assumes panel merging has already happened.
    """

    # Optionally enable ESI rendering in tiles that support this
    if not request.getHeader(ESI_HEADER):
        registry = queryUtility(IRegistry)
        if registry is not None:
            if registry.forInterface(IBlocksSettings).esi:
                request.environ[ESI_HEADER_KEY] = 'true'

    root = tree.getroot()
    headNode = root.find('head')
    baseURL = request.getURL()
    if request.getVirtualRoot():
        # plone.subrequest deals with VHM requests
        baseURL = ''
    for tileNode in utils.headTileXPath(tree):
        tileHref = tileNode.attrib[utils.tileAttrib]
        if not tileHref.startswith('/'):
            tileHref = urljoin(baseURL, tileHref)
        try:
            tileTree = utils.resolve(tileHref)
        except RuntimeError:
            tileTree = None
        except NotFound:
            continue
        if tileTree is not None:
            tileRoot = tileTree.getroot()
            utils.replace_with_children(tileNode, tileRoot.find('head'))

    for tileNode in utils.bodyTileXPath(tree):
        tileHref = tileNode.attrib[utils.tileAttrib]
        tileRulesHref = tileNode.attrib.get(utils.tileRulesAttrib)

        if not tileHref.startswith('/'):
            tileHref = urljoin(baseURL, tileHref)
        try:
            tileTree = utils.resolve(tileHref)
        except RuntimeError:
            tileTree = errorTile(request)
        except NotFound:
            continue

        if tileRulesHref:
            if not tileRulesHref.startswith('/'):
                tileRulesHref = urljoin(baseURL, tileRulesHref)
            try:
                tileTransform = resolve_transform(tileRulesHref, tileNode)
            except NotFound:
                tileTransform = None
            del tileNode.attrib[utils.tileRulesAttrib]
        else:
            tileTransform = None

        if tileTree is not None:
            tileRoot = tileTree.getroot()

            tileHead = tileRoot.find('head')
            tileBody = tileRoot.find('body')

            if tileHead is None and tileBody is None:
                tileBody = tileRoot

            if tileTransform is not None:
                result = tileTransform(tileBody).getroot()
                del tileBody[:]
                tileBody.append(result)

            if tileHead is not None:
                for tileHeadChild in tileHead:
                    headNode.append(tileHeadChild)
            utils.replace_with_children(tileNode, tileBody)

    return tree



# -*- coding: utf-8 -*-
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.app.blocks.interfaces import IBlocksTransformEnabled
from plone.app.blocks.interfaces import ILayoutField
from plone.app.blocks.interfaces import ILayoutFieldDefaultValue
from plone.app.blocks.interfaces import IOmittedField
from plone.app.blocks.interfaces import _
from plone.app.layout.globals.interfaces import IViewView
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.browser.view import DefaultView
from plone.outputfilters import apply_filters
from plone.outputfilters.interfaces import IFilter
from zope import schema
from zope.component import adapter, getMultiAdapter
from zope.component import getAdapters
from zope.component.hooks import getSite
from zope.globalrequest import getRequest
from zope.interface import Interface
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.interface import implements, provider
from zope.schema.interfaces import IContextAwareDefaultFactory
import logging
import os

try:
    from plone.supermodel import model
    from plone.supermodel.directives import fieldset
except ImportError:
    # BBB: Plone 4.2 with Dexterity 1.x
    from plone.directives import form as model
    from plone.directives.form import fieldset

try:
    from collective.dexteritytextindexer import searchable
    HAS_DXTEXTINDEXER = True
except ImportError:
    HAS_DXTEXTINDEXER = False


logger = logging.getLogger('plone.app.blocks')


class LayoutField(schema.Text):
    """A field used to store layout information
    """

    implements(ILayoutField)


@implementer(ILayoutFieldDefaultValue)
@adapter(Interface, Interface)
def layoutFieldDefaultValue(context, request):
    return u"""\
<!DOCTYPE html>
<html lang="en" data-layout="./@@page-site-layout">
<body>
<div data-panel="content">
</div>
</body>
</html>"""


@provider(IContextAwareDefaultFactory)
def layoutFieldDefaultValueFactory(context):
    if context is None:
        context = getSite()
    request = getRequest()
    return getMultiAdapter((context, request), ILayoutFieldDefaultValue)


class ILayoutAware(model.Schema):
    """Behavior interface to make a type support layout.
    """
    if HAS_DXTEXTINDEXER:
        searchable('content')
    content = LayoutField(
        title=_(u"Custom layout"),
        description=_(u"Custom content and content layout of this page"),
        defaultFactory=layoutFieldDefaultValueFactory,
        required=False
    )

    pageSiteLayout = schema.Choice(
        title=_(u"Site layout"),
        description=_(u"Site layout to apply to this page "
                      u"instead of the default site layout"),
        vocabulary="plone.availableSiteLayouts",
        required=False
    )

    sectionSiteLayout = schema.Choice(
        title=_(u"Section site layout"),
        description=_(u"Site layout to apply to sub-pages of this page "
                      u"instead of the default site layout"),
        vocabulary="plone.availableSiteLayouts",
        required=False
    )

    fieldset('layout', label=_('Layout'),
             fields=('content', 'pageSiteLayout', 'sectionSiteLayout'))

alsoProvides(ILayoutAware, IFormFieldProvider)

alsoProvides(ILayoutAware['content'], IOmittedField)
alsoProvides(ILayoutAware['pageSiteLayout'], IOmittedField)
alsoProvides(ILayoutAware['sectionSiteLayout'], IOmittedField)


class SiteLayoutView(BrowserView):
    """Default site layout view called from the site layout resolving view"""

    implements(IViewView)

    index = ViewPageTemplateFile(os.path.join('templates',
                                              'main_template.pt'))

    def __call__(self):
        self.__name__ = 'main_template'
        return self.index()


class ContentLayoutView(DefaultView):
    """Default view for a layout aware page
    """

    implements(IBlocksTransformEnabled)

    def __init__(self, context, request):
        super(ContentLayoutView, self).__init__(context, request)

    def __call__(self):
        """Render the contents of the "content" field coming from
        the ILayoutAware behavior.

        This result is supposed to be transformed by plone.app.blocks.
        """
        layout = ILayoutAware(self.context).content
        # Here we skip legacy portal_transforms and call plone.outputfilters
        # directly by purpose
        filters = [f for _, f
                   in getAdapters((self.context, self.request), IFilter)]
        return apply_filters(filters, layout)
